import operator
from musicsort_util import text_prompt, value_prompt, list_and_text_prompt

BITRATE_DIVISOR = 8000
VARIOUS_ARTISTS = u'VA'

def get_final_bitrate_info(bitrates):
    min_bitrate = min(bitrates)
    max_bitrate = max(bitrates)
    if min_bitrate == max_bitrate:
        # Constant bitrate
        return unicode(str(max_bitrate)[:-3])

    # This is a bit dodgy, really only appropriate for MP3.
    # Check if all bitrates are divisible by 8000.
    elif reduce(lambda x, y: x == y and y == True, map(lambda x: x % BITRATE_DIVISOR == 0, bitrates)):
        # Mixed bitrates
        return u"mixed, %s-%s" % (unicode(str(min_bitrate)[:-3]), unicode(str(max_bitrate)[:-3]))

    # Otherwise, assume it's variable bitrate.
    else:
        avg_bitrate = reduce(lambda x, y: x+y, bitrates) / len(bitrates)
        return u"vbr %s" % unicode(str(avg_bitrate)[:-3])

def get_final_type_info(music_types):
    # First, sort the types by # of occurrences
    counts = dict.fromkeys(music_types, 0)
    for mt in music_types:
        counts[mt] += 1
    types_s = [item for item in sorted(counts, key=operator.itemgetter(1), reverse=True)]
    # Join all types
    return u', '.join(types_s)

def check_for_missing_values(album_data, manual_values_required):
    # Prompt user to specify custom value for each missing value.
    values_to_check = ['artist', 'year', 'album']
    for vtc in values_to_check:
        if manual_values_required[vtc]:
            if len(album_data[vtc]) == 0:
                val = text_prompt("Missing value for %s, please specify custom value." % vtc)
            elif vtc == 'artist': # hack for Various Artists support
                val = VARIOUS_ARTISTS
            else:
                val = list_and_text_prompt("Multiple values for '%s'. Choose from below or enter custom text." % vtc, album_data[vtc])
            album_data[vtc] = val
        else:
            # Assume that if a manual value is not required, we always have
            #  a list of length 1.
            album_data[vtc] = album_data[vtc][0]

def get_override_values(album_data):
    od_list = sorted(album_data.keys())
    for od_key in od_list:
        if od_key == 'aux':
            continue
        val = text_prompt("%s (%s):" % (od_key, album_data[od_key]))
        if not (val is None or val == ""):
            album_data[od_key] = val

def get_aux_info(album_data):
    val = text_prompt("%s (%s):" % ('aux', album_data['aux']))
    if not (val is None or val == ""):
        album_data['aux'] = val
    else:
        album_data['aux'] = None

def construct_destination_name(album_data):
    out_dir_name = "%s - %s - %s %s(%s %s)" % (
        album_data['artist'],
        album_data['year'],
        album_data['album'],
        '' if album_data['aux'] is None else u'[' + album_data['aux'] + u'] ',
        album_data['mtype'],
        album_data['bitrate']
    )
    return sanitize_filename(out_dir_name)

def sanitize_filename(in_name):
    table = {
        ord(u'*') : u'-',
        ord(u'\\') : u'-',
        ord(u'/') : u'-',
        ord(u'?') : u' ',
        ord(u':') : u' -',
        ord(u'"') : u"'",
        ord(u'<') : u'[',
        ord(u'>') : u']',
        ord(u'|') : u'-',
    }
    return in_name.translate(table)

def get_output_name(album_data, manual_values_required):
    check_for_missing_values(album_data, manual_values_required)
    final_bitrate = get_final_bitrate_info(album_data['bitrate'])
    final_type = get_final_type_info(album_data['mtype'])
    album_data['bitrate'] = final_bitrate # hmmm, a bit dodgy changing the type
    album_data['mtype'] = final_type

    out_dir_name = construct_destination_name(album_data)

    while True: # allow user to confirm manual overrides by looping
        pval = value_prompt("Output dir will be called this, proceed? (Y)es/(N)o/(M)anual override/(A)uxilliary info\n%s" % out_dir_name,
                ["y", "yes", "m", "manual", "manual override", "a", "aux", "auxilliary info"])
        if pval == "m" or pval == "manual" or pval == "manual override":
            get_override_values(album_data)
            out_dir_name = construct_destination_name(album_data)
            print "-"
        elif pval == "a" or pval == "aux" or pval == "auxilliary info":
            get_aux_info(album_data)
            out_dir_name = construct_destination_name(album_data)
            print "-"
        elif pval == "y" or pval == "yes":
            return out_dir_name
        else:
            return None
