import logging
import os
import shutil

from musicsort_util import value_prompt, text_prompt
from musicsort_util import AlreadyExistsError

def get_genre_dirs(in_path):
    dirs = [d for d in os.listdir(in_path) if os.path.isdir(os.path.join(in_path, d)) and not d.startswith('_')]
    dirs = sorted(dirs)
    return dirs

def get_destination(music_root, out_dir_name):
    genre_dirs = get_genre_dirs(music_root)
    print u"Genre dirs: "
    for i, gd in enumerate(genre_dirs):
        print u"  %s. %s" % (str(i + 1), gd)
    while True:
        dir_val = value_prompt(u"Which number dir should this go in?", [unicode(str(i)) for i in range(1, len(genre_dirs) + 1)])
        if dir_val != None:
            dir_val = int(dir_val) - 1
            break
        print u"Invalid entry."
    final_out_dir = (
        os.path.join(
            music_root,
            genre_dirs[dir_val],
            out_dir_name
        )
    )

    # Bit of a hack to work around duplicate desination dirs if the desired
    #  desination already exists.
    aux_id = ''
    while os.path.exists(final_out_dir + aux_id):
        aux_id = text_prompt(u"Destination dir already exists. Enter aux ID, or leave blank to skip this directory.")
        if aux_id is None or aux_id == '':
            return None
    if aux_id != '':
        final_out_dir += u" [%s]" % aux_id
    return final_out_dir

def move_to_destination(source, destination):
    logging.debug(u"Moving to %s..." % destination)
    try:
        shutil.move(source, destination)
    except WindowsError, we:
        if we.winerror == 183 or we.errno == 17:
            raise AlreadyExistsError(u"Destination already exists.")
        else:
            raise we
    logging.debug(u"Finished moving")

def _test():
    logging.basicConfig(format="", level=logging.DEBUG)
    source = unicode(os.path.join(u"F:", u"temp", u"Test Album"))
    music_root = unicode(os.path.join(u"Z:", u"Lossy", u"Albums and Compilations"))
    out_dir_name = u"Test Album - 2011 - Test Album Title (MP3 vbr-179)"
    final_out_dir = get_destination(music_root, out_dir_name)
    move_to_destination(source, final_out_dir)

if __name__ == "__main__": _test()