import logging
import os
import re
import sys

FILENAME_ENCODING = "UTF-16"

class AppError(Exception):
    pass

class ArgumentError(AppError):
    pass

class NotFoundError(AppError):
    pass

class NotSupportedError(AppError):
    pass

class AlreadyExistsError(AppError):
    pass

class TagInconsistencyError(AppError):
    pass

def action_prompt(msg):
    print msg
    val = raw_input(">> ").lower().strip()

    if val == 'y' or val == 'yes':
        return True
    else:
        return False

def value_prompt(msg, values):
    print msg
    val = raw_input(">> ").lower().strip()

    if val in values:
        return val
    else:
        return None

def text_prompt(msg):
    print msg
    val = raw_input(">> ").decode(sys.stdout.encoding).strip()

    return val

def list_and_text_prompt(msg, options):
    print msg
    for i, op in enumerate(options):
        print u"  %d. %s" % (i + 1, op)

    val = raw_input(">> ").strip()

    try:
        op_i = int(val) - 1
        if op_i >= 0 and op_i < len(options):
            val = options[op_i]
    except ValueError, ve:
        # Use whatever the user entered as a custom value
        val = val.decode(sys.stdout.encoding)

    return val

def get_file_ext(in_file_name):
    return os.path.splitext(in_file_name)[1].upper().lstrip('.')

def get_all_subdirs(root_dir):
    """ Returns a list of full-path dirs contained within the input dir. If no sub-dirs, returns an empty list."""
    return [os.path.join(root_dir, d)
                for d in os.listdir(root_dir)
                if os.path.isdir(os.path.join(root_dir, d))]

def get_all_files(root_dir):
    """ Returns a list of full-path files contained within the input dir. If no sub-dirs, returns an empty list."""
    return [os.path.join(root_dir, d)
                for d in os.listdir(root_dir)
                if os.path.isfile(os.path.join(root_dir, d))]

YEAR_REGEX = re.compile('\\D*([1-2][0-9][0-9][0-9])\\D*')
def extract_year_from_text_date(in_date):
    return YEAR_REGEX.search(in_date).group(1)